/***********************************************************************/

import java.io.*;
import java.util.*;

/**
 * Classe che ingloba un file e permette di scrivere i libri di una
 * una biblioteca su file.
 */

public class BiblioWriter
{
  private PrintStream fd;        // il file di output su cui scrivere

/** Apre file con nome dato e crea writer su quel file.
 *  @param   name   nome del file da aprire in output
 */
  public BiblioWriter(String name) throws IOException
  {  fd = new PrintStream(new FileOutputStream(name));  }

/** Crea writer su file gia' aperto.
 *  @param   ffdd   file gia' aperto in output
 */
  public BiblioWriter(PrintStream ffdd)  {  fd = ffdd;  }

/** Scrive libro i-esimo di una biblioteca, ritorna true se tutto
 *  ok e false altrimenti.
 *  @param   bib   la biblioteca a cui appartiene il libro da scrivere
 *  @param   i     posizione del libro nella biblioteca
 */
  public boolean writeBook(Biblioteca bib, int i)
  {
    LibroInBiblioteca l = bib.libroPerCodice(i);
    if (l==null) return false;
    fd.print(BiblioIO.AUT + " " + l.autore + " ");
    fd.print(BiblioIO.TIT + " " + l.titolo + " ");
    fd.print(BiblioIO.NUM + " " + l.copieTotali() + " ");
    fd.println(BiblioIO.PST + " " + l.copiePrestate());
    return true;
  } 

/** Scrive sul file tutti i libri di una biblioteca. 
 *  @param   bib   la biblioteca da scrivere
 */
  public void write(Biblioteca bib)
  {
    fd.println(BiblioIO.KEY_WORD + " " + bib.laCapacita());
    int i;
    for (i=0; i<bib.numeroLibri(); i++)
    {  writeBook(bib,i);  }
  } 

  /*
  MAIN DI PROVA: 
  crea biblioteca, ci mette dei libri, la scrive sul file pippo.txt
  */
  public static void main(String[] args)
  {
    Biblioteca b = new Biblioteca();
    b.nuovoLibro("Publio Virgilio Marone", "Eneide", 1);
    b.nuovoLibro("Dante Alighieri", "La Commedia", 2);
    b.nuovoLibro("Alessandro Manzoni", "I Promessi Sposi", 3);
    b.nuovoLibro("Ludovico Ariosto", "Orlando Furioso", 1);
    try
    {
      BiblioWriter wr = new BiblioWriter("pippo.txt");
      wr.write(b);
    }
    catch(Exception exc) {};
  }
};

/***********************************************************************/
