/**
 * Classe che implementa la biblioteca:
 * contiene un insieme di libri memorizzati in un array e internamente 
 * identificati con la loro posizione in questo array (posizione che
 * peraltro puo' cambiare nel tempo in seguito ad acquisizione e 
 * dismissione di libri);
 * la biblioteca puo' contenere un numero massimo di libri,
 * detto capacita' della biblioteca.
 */

public class Biblioteca // implements Serializable
{
    private int capacita;              // quanti libri puo' contenere
    private LibroInBiblioteca[] libri; // array di libri
    private int numero;                // contatore libri presenti

    /* COSTRUTTORI */

/** Costruisce biblioteca con capacita' convenzionale = 100. */
    public Biblioteca()
    {
      capacita = 100;
      libri = new LibroInBiblioteca[100];
      numero = 0;
    }

/** Costruisce biblioteca con capacita' data.
 *  @param   cap   capacita' della nuova biblioteca, e' ignorato e 
 *                 sostituito dal valore 100 se minore di 100
 */
    public Biblioteca(int cap)
    {
      capacita = (cap>=100) ? cap : 100;
      libri = new LibroInBiblioteca[capacita];
      numero = 0;
    }

    /* METODI CHE INTERROGANO LO STATO ATTUALE DELLA BIBLIOTECA*/

/** Fornisce numero massimo di libri (capacita' della biblioteca). */
    public int laCapacita() {  return capacita;  }

/** Fornisce numero di libri presenti in biblioteca. */
    public int numeroLibri() { return numero; }

/** Restituisce libro dato il suo codice
 *  @param   cod   codice del libro ovvero sua posizione nell'array
 */
    public LibroInBiblioteca libroPerCodice(int cod)
    {  
       if ((cod<0) || (cod>numero)) return null;
       else return libri[cod];
    }

    /* METODI CHE MODIFICANO LO STATO ATTUALE DELLA BIBLIOTECA */

/** Cambia la capacita', eseguito solo se la biblioteca e' senza libri.
 *  @param   cap   la nuova capacita', e' ignorato e
 *                 sostituito dal valore 100 se minore di 100
 *  @return  true se l'operazione e' stata eseguita (= se la
 *           biblioteca non ha libri), false altrimenti
 */
    boolean cambiaCapacita(int cap)
    {
      if (numero>0) return false;
      capacita = (cap>=100) ? cap : 100;
      libri = new LibroInBiblioteca[capacita];
      return true;
    }

/** Aggiunge alla biblioteca un nuovo libro, in un certo numero di copie,
 *  solo se il numero di libri presenti non ha raggiunto la capacita'.
 *  @param   aut   autore del nuovo libro
 *  @param   tit   titolo del nuovo libro
 *  @param   num   numero di copie del nuovo libro
 *  @return  true in caso di successo, false altrimenti
 */
    public boolean nuovoLibro(String aut, String tit, int num)
    {
       if (numero<capacita)
       {
         libri[numero++] = new LibroInBiblioteca(aut,tit,num);
         return true;
       }
       return false; // non c'era un posto libero
    }

/** Fa rientrare una copia prestata del libro, se qualche copia e'
 *  stata prestata.
 *  @param   cod   codice del libro ovvero sua posizione nell'array
 */
    public void restitLibro(int cod) throws Exception
    {
      LibroInBiblioteca presente = libri[cod];
      if (presente == null)
        throw new Exception("Restituzione: codice libro non valido");
      else
        if (!presente.restituisci())
           throw new Exception("Restituzione: libro non prestato");
    }

/** Fa uscire in prestito una copia del libro, se non tutte le copie
 *  sono state prestate.
 *  @param   cod   codice del libro ovvero sua posizione nell'array
 */
    public void prestaLibro(int cod) throws Exception
    {
      LibroInBiblioteca presente = libri[cod];
      if (presente == null)
        throw new Exception("Prestito: codice libro non valido");
      else
      {
        if (!presente.presta())
          throw new Exception("Prestito: non ci sono copie disponibili");
      }
    }

/** Elimina libro dalla biblioteca, anche se ha copie prestate.
 *  @param   cod   codice del libro ovvero sua posizione nell'array
 *  @return  true in caso di successo, false altrimenti
 */
    public boolean dismettiLibro(int cod)
    {
      LibroInBiblioteca presente = libri[cod];
      if (presente != null)
      {
        libri[cod] = libri[--numero];
        return true;
      }
      return false; // codice libro non era valido
    }
}
