import java.awt.*;
import java.awt.event.*;

/*
Finestra di dialogo con tre bottoni Ok, Canc nella parte inferiore,
nella parte centrale pannello con quattro campi di testo per
codice, autore, titolo, numero copie di un libro
(ogni campo di testo e' accompagnato da un'etichetta esplicativa),
e nella parte superiore bottoni per nuoversi avanti/indietro
nella lista di libri.
*/

public class Dialog3 extends Dialog2
{
/** bottone per andare all'inizio della lista di libri */
  protected Button bPrimo;
/** bottone per andare in fondo della lista di libri */
  protected Button bUltimo;
/** bottone per andare avanti uno nella lista di libri */
  protected Button bAvanti;
/** bottone per andare indietro uno nella lista di libri */
  protected Button bIndietro;

/** Codice del libro correntemente mostrato. */
  protected int codiceCorrente = -1; 

/** Biblioteca a cui appartiene il libro di cui sopra. */
  protected Biblioteca biblioteca;

/**
 * Costruisce la finestra di dialogo dal punto 
 * di vista del contenuto grafico, senza comportamento.
 */
  protected void costruisci()
  {
    super.costruisci();
    // rende non editabili i campi di testo
    tAut.setEditable(false);
    tTit.setEditable(false);
    tCop.setEditable(false);
    // recupera etichetta delle copie e cambia il testo
    Panel pSx = (Panel)pCentro.getComponent(0);
    Label lCop = (Label)pSx.getComponent(2);
    lCop.setText("Copie (in prestito):");
    // crea pannello superiore
    pSopra = new Panel(new GridLayout(1,4));
    pSopra.add(bPrimo = new Button(" |<< "));
    pSopra.add(bIndietro = new Button("  <  "));
    pSopra.add(bAvanti = new Button("  >  "));
    pSopra.add(bUltimo = new Button(" >>| "));
    add(pSopra,BorderLayout.NORTH);
  }

/** Ridefinita per tornare a inizio lista prima di rendere visibile */
  public void setVisible(boolean v)
  {  
     if (v) indietroTutta();
     super.setVisible(v);
  }

/**
 * Costruisce finestra di dialogo dati la finestra da cui dipende,
 * il titolo da mettere nella barra del titolo, la biblioteca,
 * flag se la finestra di dialogo sia modale on no.
 * @see Dialog3#costruisci
 */
  public Dialog3(Frame f, String title, Biblioteca bib, boolean modal)
  {
    super(f,title,modal);
    biblioteca = bib;
    bPrimo.addActionListener(new ActionListener()
    {
       public void actionPerformed(ActionEvent e)
       { indietroTutta(); }
    });
    bIndietro.addActionListener(new ActionListener()
    {
       public void actionPerformed(ActionEvent e)
       { vaiIndietro(); }
    });
    bAvanti.addActionListener(new ActionListener()
    {
       public void actionPerformed(ActionEvent e)
       { vaiAvanti(); }
    });
    bUltimo.addActionListener(new ActionListener()
    {
       public void actionPerformed(ActionEvent e)
       { avantiTutta(); }
    });
  }

/**
 * Riempie i campi di testo coi dati del libro di codice dato,
 * se esiste; se biblioteca non esiste o codice non corrisponde a
 * un libro svuota tutti i campi di testo.
 * @param   cod   codice, nella biblioteca, del libro da mostrare 
 */
  public void mostraLibro(int cod)
  {
    LibroInBiblioteca l = null;
    if (biblioteca!=null) l = biblioteca.libroPerCodice(cod);
    if (l==null) pulisci();
    else
    {  codiceCorrente = cod;
       tAut.setText(l.autore); tTit.setText(l.titolo); 
       tCop.setText(l.copieTotali() + " (" + l.copiePrestate() + ")"); 
    }
  }

/** Ritorna codice del libro corrente, se esiste, altrimenti -1 */
  public int codiceScelto() {  return codiceCorrente;  }

/** Scorre avanti di uno nella lista dei libri. */
  void vaiAvanti() 
  {  
    if (codiceCorrente<biblioteca.numeroLibri()-1)  codiceCorrente++;
    mostraLibro(codiceCorrente);
  }

/** Scorre indietro di uno nella lista dei libri. */
  void vaiIndietro() 
  {  
    if (codiceCorrente>0)  codiceCorrente--;
    mostraLibro(codiceCorrente);
  }

/** Va avanti fino all'ultimo libro della lista dei libri. */
  void avantiTutta() 
  {  
    codiceCorrente = biblioteca.numeroLibri()-1;
    mostraLibro(codiceCorrente);
  }

/** Va indietro fino al primo libro della lista dei libri. */
  void indietroTutta() 
  {  
    if ( (biblioteca!=null) && (biblioteca.numeroLibri()>0) )
       codiceCorrente = 0;
    else codiceCorrente = -1;
    mostraLibro(codiceCorrente);
  }

/** Funzione main di prova. */
  public static void main(String[] args)
  {
    /* crea frame f con bottone b */
    final Frame f = new Frame();
    final Button b = new Button("Apri Dialog 3");
    f.add(b);
    f.pack();
    /* crea dialogo d dipendente da f su biblioteca nuova */
    final Dialog3 d = new Dialog3(f,"Dialog 3",new Biblioteca(),true);
    /* bottone b su frame f mostra dialogo d */
    b.addActionListener(d.showListener());
    /* bottone ok su d stampa stringa e nasconde d */
    d.addOkListener(new ActionListener()
      {  public void actionPerformed(ActionEvent e)
         {  System.out.println("Libro = " + d.codiceScelto());
            d.setVisible(false);
         }
      });
    /* bottone canc su d nasconde d */
    d.addCancListener(d.hideListener());
    /* metti qualche libro nella biblioteca associata a d */
    d.biblioteca.nuovoLibro("Manzoni", "Promessi sposi", 2);
    d.biblioteca.nuovoLibro("Dante", "Commedia", 2);
    d.biblioteca.nuovoLibro("Omero", "Odissea", 2);
    d.biblioteca.nuovoLibro("Virgilio", "Eneide", 2);
    /* mostra a schermo il frame */
    f.setVisible(true);
  }

}


